package com.menutoday;


import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.concurrent.TimeUnit;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.openqa.selenium.By;
import org.openqa.selenium.Keys;
import org.openqa.selenium.NoSuchElementException;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.chrome.ChromeDriver;

/**
 * 
 * @author igomezuoc
 *
 */

public class TestSelenium {
	private WebDriver driver;
	private String baseUrl;


	@Before
	public void setUp() throws Exception {
		String path = System.getenv("MENUTODAY_CONFIG_FILE");
		Properties prop = new Properties();
		InputStream in = new FileInputStream(new File(path));
		prop.load(in);
		in.close();
		System.setProperty ("webdriver.chrome.driver", prop.getProperty("webdriver.chrome.driver"));
		driver = new ChromeDriver();
		baseUrl = "http://localhost:8080/menuapp";
		driver.manage().timeouts().implicitlyWait(2, TimeUnit.SECONDS);
	}

	/**
	 * Test Case: Realitzar login a l'aplicaci de restaurants mitjanant un usuari final
	 * 
	 * Resultat: Accedir a la pantalla incial dels usuaris i comprovar si es mostren les opcions de men
	 * @throws Exception
	 */
	@Test
	public void testLogin () throws Exception {
		// login
		driver.get(baseUrl);
		driver.findElement(By.name("j_username")).sendKeys("u");
		driver.findElement(By.name("j_password")).sendKeys("u");
		driver.findElement(By.name("j_password")).sendKeys(Keys.ENTER);
		assertTrue(driver.getCurrentUrl().contains("menuapp/menutoday/master"));
		
		// obtenim les opcions de menu
		WebElement div = driver.findElement(By.className("list-group"));
		WebElement e = div.findElement(By.xpath(".//*[@href='/menuapp/menutoday/users/meus-restaurants']"));
		WebElement e2 = div.findElement(By.xpath(".//*[@href='/menuapp/menutoday/users/reserva-confirmada']"));
		WebElement e4 = div.findElement(By.xpath(".//*[@href='/menuapp/menutoday/users/promocions']"));
		WebElement e5 = div.findElement(By.xpath(".//*[@href='/menuapp/menutoday/j_spring_security_logout']"));
		assertEquals(true, e.isDisplayed());
		assertEquals(true, e2.isDisplayed());
		assertEquals(true, e4.isDisplayed());
		assertEquals(true, e5.isDisplayed());
	}

	/**
	 * Test Case: Aprovar les sollicituts pendents que han enviat a la "Company A"
	 * 
	 * Resultat: Sollicituts aprovades i no hi ha sollicituts pendents per a confirmar
	 *  
	 * @throws Exception
	 */
	@Test
	public void testAprovarSolicituds () throws Exception {
		//login
		driver.get(baseUrl);
		driver.findElement(By.name("j_username")).sendKeys("a");
		driver.findElement(By.name("j_password")).sendKeys("a");
		driver.findElement(By.name("j_password")).sendKeys(Keys.ENTER);
		assertTrue(driver.getCurrentUrl().contains("menuapp/menutoday/master"));
		driver.get(baseUrl + "/menutoday/solicituds/rebre");

		// Accs a la pgina correcte
		WebElement aTitle = driver.findElement(By.className("navbar-brand"));
		assertEquals("Sollicituds rebudes", aTitle.getText());
	
		// Comprovaci 3 solicituts pendents de confirmar
		assertEquals(3, driver.findElements(By.className("my-checkbox")).size());
		
		// Acceptem totes 3
		for (WebElement checkbox : driver.findElements(By.className("my-checkbox"))) {
			checkbox.click();
		}
		
		// Confirmar
		driver.findElement(By.id("confirmar")).click();
		
		// Ja no tenim peticions pendents de confirmar
		driver.get(baseUrl + "/menutoday/solicituds/rebre");
		assertEquals(0,  driver.findElements(By.className("my-checkbox")).size() );
	}

	/**
	 * Test Case: Comprovar que un usuari de la "Company A" t acces als nous menus
	 * 
	 * Resultat: L'usuari pot accedir als nous restaurants desde "Meus Restaurants"
	 * 
	 * @throws Exception
	 */
	@Test
	public void testMeusRestaurants () throws Exception {
		// login
		driver.get(baseUrl);
		driver.findElement(By.name("j_username")).sendKeys("u");
		driver.findElement(By.name("j_password")).sendKeys("u");
		driver.findElement(By.name("j_password")).sendKeys(Keys.ENTER);
		assertTrue(driver.getCurrentUrl().contains("menuapp/menutoday/master"));
		driver.get(baseUrl + "/menutoday/users/meus-restaurants");

		// Obtenir el numero de restaurants
		List<WebElement> list = driver.findElements (By.xpath(".//*[@class='panel-heading']"));
		assertEquals (5, list.size());
		
		// Comprobem que es trobem tots els noms de restaurants que hem acceptat a les solicituds
		List<String> result = new ArrayList<String>();
		for (WebElement element : list) {
			result.add(element.findElement(By.tagName("b")).getText());
		}
		
		// comparem resultats esperats amb els obtinguts
		String [] expectedResult = {"Casa lvaro","La Ginesa","Pizza SI","Restaurant MUN cuina evocativa","Vinya Nova"};
		for (int z=0; z<expectedResult.length;z++) {
			assertTrue(result.contains(expectedResult[z]));
		}
	}
	
	/**
	 * Test Case: Poder realitzar una reserva en un dels restaurants vinculats a l'empresa
	 * 
	 * Resultat: La reserva es confirma i podem accedir a la reserva desde el men
	 * @throws Exception
	 */
	@Test 
	public void testReserva () throws Exception {
		// login
		driver.get(baseUrl);
		driver.findElement(By.name("j_username")).sendKeys("u");
		driver.findElement(By.name("j_password")).sendKeys("u");
		driver.findElement(By.name("j_password")).sendKeys(Keys.ENTER);
		assertTrue(driver.getCurrentUrl().contains("menuapp/menutoday/master"));
		
		// accedir als meus restaurants
		WebElement div = driver.findElement(By.className("list-group"));
		WebElement e = div.findElement(By.xpath(".//*[@href='/menuapp/menutoday/users/meus-restaurants']"));
		e.click();
		
		// Seleccionar el primer restaurant
		List<WebElement> list = driver.findElements (By.xpath(".//*[@class='panel-heading']"));
		list.get(0).click();
		
		// Realitzar la reserva
		driver.findElement(By.id("bt-reserva")).click();
		
		// Accedir a la confirmacio de reserva
		driver.get(baseUrl + "/menutoday/users/reserva-confirmada/");
		
		// busquem l'alerta reserva no trobada i en cas de no trobar l'element significa que tenim una reserva
		try {
			driver.findElement(By.className("bs-reserva")).
				findElement(By.xpath(".//*[@class='alert alert-info']"));
			fail ("Reserva no trobada");
		} catch (NoSuchElementException nse) {
			assertTrue(nse.getMessage().contains("no such element"));
		}
	}


	@After
	public void tearDown() throws Exception {
		driver.quit();
	}

}